import { combineLatestOrDefault } from '@adminide-stack/extension-api';
import { combineLatest, from, map, distinctUntilChanged, switchMap, of, catchError } from 'rxjs';
import { isEqual } from 'lodash-es';
import { memoizeObservable } from '../util/memorize-observable.js';
import { getScriptURLFromExtensionManifest, viewerConfiguredExtensions, isExtensionEnabled } from '../extensions/extension.js';
import { isDefined } from '../util/types.js';

// Fetch sideloaded extensions function
async function fetchSideloadedExtension(baseUrl) {
  try {
    const response = await fetch(`${baseUrl}/package.json`);
    if (!response.ok) {
      throw new Error(`Request failed with status ${response.status}`);
    }
    const data = await response.json();
    return {
      id: data.name,
      manifest: {
        ...data,
        url: `${baseUrl}/${data.main.replace('dist/', '')}`
      },
      rawManifest: null
    };
  } catch (error) {
    console.error(error, 'Error fetching sideloaded extension');
    return null;
  }
}
// Function to create an extensions service object
function createExtensionsService(platformContext, modelService) {
  // Memoized function to fetch script URLs
  const memoizedGetScriptURLForExtension = memoizeObservable(url => asObservable(platformContext.getScriptURLForExtension(url)).pipe(catchError(error => {
    console.error(error, `Error fetching extension script URL ${url}`);
    return of(null);
  })), url => url);
  // Function to get enabled extensions
  function getEnabledExtensions() {
    return combineLatest([from(platformContext.settings), from(viewerConfiguredExtensions({
      settings: platformContext.settings,
      requestGraphQL: platformContext.requestGraphQL
    })), getSideloadedExtension()]).pipe(map(([settings, configuredExtensions, sideloadedExtension]) => {
      let enabled = configuredExtensions.filter(extension => isExtensionEnabled(settings, extension.id));
      if (sideloadedExtension) {
        enabled.push(sideloadedExtension);
      }
      return enabled;
    }));
  }
  // Function to get sideloaded extensions
  function getSideloadedExtension() {
    return from(platformContext.sideloadedExtensionURL).pipe(switchMap(url => url ? fetchSideloadedExtension(url) : of(null)), catchError(error => {
      console.error(error, 'Error sideloading extension');
      return of(null);
    }));
  }
  // Function to return active extensions
  function getActiveExtensions() {
    const activatedExtensionIDs = new Set();
    return combineLatest([from(modelService.activeLanguages), getEnabledExtensions()]).pipe(map(([activeLanguages, enabledExtensions]) => {
      const activeExtensions = extensionsWithMatchedActivationEvent(enabledExtensions, activeLanguages);
      for (const extension of activeExtensions) {
        if (!activatedExtensionIDs.has(extension.id)) {
          activatedExtensionIDs.add(extension.id);
        }
      }
      return activeExtensions;
    }), distinctUntilChanged((a, b) => isEqual(new Set(a.map(ext => ext.id)), new Set(b.map(ext => ext.id)))), switchMap(extensions => combineLatestOrDefault(extensions.map(extension => memoizedGetScriptURLForExtension(getScriptURLFromExtensionManifest(extension)).pipe(map(scriptURL => scriptURL ? {
      id: extension.id,
      manifest: extension.manifest,
      scriptURL
    } : null))))), map(extensions => extensions.filter(isDefined)), distinctUntilChanged((a, b) => isEqual(new Set(a.map(ext => ext.id)), new Set(b.map(ext => ext.id)))));
  }
  return {
    activeExtensions: getActiveExtensions()
  };
}
// Helper function to return observables
function asObservable(input) {
  return typeof input === 'string' ? of(input) : from(input);
}
// Function to filter extensions with activation events
function extensionsWithMatchedActivationEvent(enabledExtensions, visibleTextDocumentLanguages) {
  const languageActivationEvents = new Set([...visibleTextDocumentLanguages].map(language => `onLanguage:${language}`));
  return enabledExtensions.filter(extension => {
    try {
      if (!extension.manifest || isEqual(extension.manifest.activationEvents, [])) {
        console.warn(`Extension ${extension.id} has no activation events.`);
        return false;
      }
      return extension.manifest.activationEvents.some(event => event === '*' || languageActivationEvents.has(event));
    } catch (error) {
      console.error(error);
    }
    return false;
  });
}

export { createExtensionsService };
//# sourceMappingURL=extensions-service.js.map
